@@ -18,6 +18,7 @@ import contextlib
 import copy
 import datetime
 import json
+import operator
 
 from automaton import exceptions as automaton_errors
 from ironicclient import exceptions
@@ -30,7 +31,6 @@ from oslo_utils import timeutils
 from oslo_utils import uuidutils
 import six
 from sqlalchemy.orm import exc as orm_errors
-from sqlalchemy import text
 
 from ironic_inspector.common.i18n import _
 from ironic_inspector.common import ironic as ir_utils
@@ -409,13 +409,14 @@ class NodeInfo(object):
         else:
             self._ports[mac] = port
 
-    def patch(self, patches, ironic=None):
+    def patch(self, patches, ironic=None, **kwargs):
         """Apply JSON patches to a node.
 
         Refreshes cached node instance.
 
         :param patches: JSON patches to apply
         :param ironic: Ironic client to use instead of self.ironic
+        :param kwargs: Arguments to pass to ironicclient.
         :raises: ironicclient exceptions
         """
         ironic = ironic or self.ironic
@@ -426,7 +427,7 @@ class NodeInfo(object):
                 patch['path'] = '/' + patch['path']
 
         LOG.debug('Updating node with patches %s', patches, node_info=self)
-        self._node = ironic.node.update(self.uuid, patches)
+        self._node = ironic.node.update(self.uuid, patches, **kwargs)
 
     def patch_port(self, port, patches, ironic=None):
         """Apply JSON patches to a port.
@@ -470,6 +471,28 @@ class NodeInfo(object):
             ironic=ironic,
             capabilities=ir_utils.dict_to_capabilities(existing))
 
+    def add_trait(self, trait, ironic=None):
+        """Add a trait to the node.
+
+        :param trait: trait to add
+        :param ironic: Ironic client to use instead of self.ironic
+        """
+        ironic = ironic or self.ironic
+        ironic.node.add_trait(self.uuid, trait)
+
+    def remove_trait(self, trait, ironic=None):
+        """Remove a trait from the node.
+
+        :param trait: trait to add
+        :param ironic: Ironic client to use instead of self.ironic
+        """
+        ironic = ironic or self.ironic
+        try:
+            ironic.node.remove_trait(self.uuid, trait)
+        except exceptions.NotFound:
+            LOG.debug('Trait %s is not set, cannot remove', trait,
+                      node_info=self)
+
     def delete_port(self, port, ironic=None):
         """Delete port.
 
@@ -733,7 +756,7 @@ def _delete_node(uuid, session=None):
     with db.ensure_transaction(session) as session:
         db.model_query(db.Attribute, session=session).filter_by(
             node_uuid=uuid).delete()
-        for model in (db.Option, db.Node):
+        for model in (db.Option, db.IntrospectionData, db.Node):
             db.model_query(model,
                            session=session).filter_by(uuid=uuid).delete()
 
@@ -817,14 +840,11 @@ def find_node(**attributes):
 
         LOG.debug('Trying to use %s of value %s for node look up',
                   name, value)
-        value_list = []
-        for v in value:
-            value_list.append("name='%s' AND value='%s'" % (name, v))
-        stmt = ('select distinct node_uuid from attributes where ' +
-                ' OR '.join(value_list))
-        rows = (db.model_query(db.Attribute.node_uuid).from_statement(
-            text(stmt)).all())
-        found.update(row.node_uuid for row in rows)
+        query = db.model_query(db.Attribute.node_uuid)
+        pairs = [(db.Attribute.name == name) &
+                 (db.Attribute.value == v) for v in value]
+        query = query.filter(six.moves.reduce(operator.or_, pairs))
+        found.update(row.node_uuid for row in query.distinct().all())
 
     if not found:
         raise utils.NotFoundInCacheError(_(
@@ -874,19 +894,10 @@ def find_node(**attributes):
 def clean_up():
     """Clean up the cache.
 
-    * Finish introspection for timed out nodes.
-    * Drop outdated node status information.
+    Finish introspection for timed out nodes.
 
     :return: list of timed out node UUID's
     """
-    if CONF.node_status_keep_time > 0:
-        status_keep_threshold = (timeutils.utcnow() - datetime.timedelta(
-                                 seconds=CONF.node_status_keep_time))
-        with db.ensure_transaction() as session:
-            db.model_query(db.Node, session=session).filter(
-                db.Node.finished_at.isnot(None),
-                db.Node.finished_at < status_keep_threshold).delete()
-
     timeout = CONF.timeout
     if timeout <= 0:
         return []
@@ -966,3 +977,44 @@ def get_node_list(ironic=None, marker=None, limit=None):
                                    ('started_at', 'uuid'),
                                    marker=marker, sort_dir='desc')
     return [NodeInfo.from_row(row, ironic=ironic) for row in rows]
+
+
+def store_introspection_data(node_id, introspection_data, processed=True):
+    """Store introspection data for this node.
+
+    :param node_id: node UUID.
+    :param introspection_data: A dictionary of introspection data
+    :param processed: Specify the type of introspected data, set to False
+                      indicates the data is unprocessed.
+    """
+    with db.ensure_transaction() as session:
+        record = db.model_query(db.IntrospectionData,
+                                session=session).filter_by(
+            uuid=node_id, processed=processed).first()
+        if record is None:
+            row = db.IntrospectionData()
+            row.update({'uuid': node_id, 'processed': processed,
+                        'data': introspection_data})
+            session.add(row)
+        else:
+            record.update({'data': introspection_data})
+        session.flush()
+
+
+def get_introspection_data(node_id, processed=True):
+    """Get introspection data for this node.
+
+    :param node_id: node UUID.
+    :param processed: Specify the type of introspected data, set to False
+                      indicates retrieving the unprocessed data.
+    :return: A dictionary representation of intropsected data
+    """
+    try:
+        ref = db.model_query(db.IntrospectionData).filter_by(
+            uuid=node_id, processed=processed).one()
+        return ref['data']
+    except orm_errors.NoResultFound:
+        msg = _('Introspection data not found for node %(node)s, '
+                'processed=%(processed)s') % {'node': node_id,
+                                              'processed': processed}
+        raise utils.IntrospectionDataNotFound(msg)
